﻿using Silanis.ESL.SDK;
using Silanis.ESL.SDK.Builder;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace FeatureGuideCharp
{
    class _05_23_notary_net_sdk
    {
        private String API_KEY = "your_api_key";
        private String API_URL = "https://sandbox.esignlive.com/api";
        private String notaryRoleId = "notary";
        private String notaryDocumentId = "document1";
        private String notaryEmail = "notary_email";
        private String signerEmail = "signer1@mailinator.com";

        public void Execute() {

            EslClient eslClient = new EslClient(API_KEY, API_URL);

            try
            {
                //step1: set signers, upload document add signatures and normal fields for non-notary signers and set notarized setting when package creation (by SDK)
                DocumentPackage pkg1 = PackageBuilder.NewPackageNamed("Create Notary Package by .NET SDK - " + System.DateTime.Now)
                                                               .WithSigner(SignerBuilder.NewSignerWithEmail(notaryEmail)
                                                                       .WithFirstName("Duo")
                                                                       .WithLastName("Liang")
                                                                       .WithCustomId(notaryRoleId)
                                                                       )
                                                               .WithSigner(SignerBuilder.NewSignerWithEmail(signerEmail)
                                                                       .WithFirstName("Marry")
                                                                       .WithLastName("Doe")
                                                                       .WithCustomId("applicant")
                                                                       )
                                                               .WithDocument(DocumentBuilder.NewDocumentNamed("document 1")
                                                                       .FromFile("your_file_path")
                                                                       .WithId(notaryDocumentId)
                                                                       .WithSignature(SignatureBuilder.SignatureFor(signerEmail).AtPosition(100,100)
                                                                            .WithField(FieldBuilder.SignatureDate().AtPosition(100,200)))
                                                                       )
                                                               .WithNotarized(true)
                                                               .Build();

                PackageId createPackageOneStep = eslClient.CreatePackageOneStep(pkg1);
                Debug.WriteLine("package id: "+ createPackageOneStep);


                //step2: set notary role id (by custom REST method)
                setNotaryRoleId(createPackageOneStep.Id, notaryRoleId);

                //step3: add signatures and notary fields for notary (by .NET SDK)
                Signature notarySignature = SignatureBuilder.SignatureFor(notaryEmail)
                         .AtPosition(400, 100)
                         .WithField(FieldBuilder.CustomField("NotaryLicenseExpiryDate")
                             .WithStyle(FieldStyle.UNBOUND_CUSTOM_FIELD)
                             .AtPosition(400,200)
                             )
                          .WithField(FieldBuilder.CustomField("NotaryName")
                             .WithStyle(FieldStyle.SEAL)
                             .AtPosition(400, 300)
                          )
                         .Build();
                eslClient.ApprovalService.AddApproval(eslClient.GetPackage(createPackageOneStep), notaryDocumentId, notarySignature);



                //step4: send package
                eslClient.SendPackage(createPackageOneStep);

        }
            catch (Exception e) {   //to do, error handling
                throw e;
            }

}

    

        private void setNotaryRoleId(String packageId, String notaryId) {
            HttpClient httpClient = new HttpClient();
            httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", API_KEY);
            httpClient.DefaultRequestHeaders.Add("Accept", "application/json");
            string updatePackageString = "{\"notaryRoleId\" : \"" + notaryId + "\"}";
            StringContent jsonUpdatePackageContent = new StringContent(updatePackageString, Encoding.UTF8, "application/json");

            var response = httpClient.PutAsync(new Uri(API_URL) + "/packages/" + packageId, jsonUpdatePackageContent).Result;
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }



    }
}
